package jalview.gui;

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import jalview.datamodel.*;
import jalview.schemes.*;
import jalview.analysis.*;


public class SeqCanvas extends JPanel
{
     FeatureRenderer fr;
    Image             img;
    Graphics          gg;
    int               imgWidth;
    int               imgHeight;

    AlignViewport     av;

    public boolean paintFlag = false;

    boolean showScores = false;
    boolean displaySearch = false;
    int [] searchResults = null;

    int chunkHeight;
    int chunkWidth;


    public SeqCanvas(AlignViewport av)
    {
	this.av         = av;
       fr = new FeatureRenderer(av);
       setLayout(new BorderLayout());
       PaintRefresher.Register(this);

    }

  public void drawScale(Graphics g, int startx, int endx,int ypos) {
      int scalestartx = startx - startx%10 + 10;

      g.setColor(Color.black);

      for (int i=scalestartx;i < endx;i+= 10)
      {
          String string = String.valueOf(i);
          g.drawString(string,(i-startx-1)*av.charWidth,ypos - av.charHeight/2);

          g.drawLine( (i-startx-1)*av.charWidth +av.charWidth/2, ypos+2 - av.charHeight/2,
                    (i-startx-1)*av.charWidth +av.charWidth/2, ypos-2 );

      }

  }


/**
 * Definitions of startx and endx (hopefully):
 * SMJS This is what I'm working towards!
 *   startx is the first residue (starting at 0) to display.
 *   endx   is the last residue to display (starting at 0).
 *   starty is the first sequence to display (starting at 0).
 *   endy   is the last sequence to display (starting at 0).
 * NOTE 1: The av limits are set in setFont in this class and
 * in the adjustment listener in SeqPanel when the scrollbars move.
 */

  public void paintComponent(Graphics g)
  {

    if (img==null ||  imgWidth!=getWidth() || imgHeight!=getHeight() || paintFlag)
    {
      imgWidth  = getWidth();
      imgHeight = getHeight();
      img = createImage(imgWidth,imgHeight);
      gg  = img.getGraphics();
      gg.setFont(av.getFont());
      paintFlag = false;
    }


    chunkWidth  =   getWidth()/av.charWidth;
    chunkHeight =  (av.getAlignment().getHeight() + 2)*av.charHeight;

    av.setChunkHeight(chunkHeight);
    av.setChunkWidth(chunkWidth);

    fillBackground(gg,Color.WHITE,0,0,imgWidth,imgHeight);

    if (av.getWrapAlignment())
      drawWrappedPanel(gg, getWidth(), getHeight(), av.startRes);
    else
      drawPanel(gg, av.startRes, av.endRes, av.startSeq, av.endSeq, av.startRes, av.startSeq, 0);


    g.drawImage(img,0,0,this);

  }

  public void drawWrappedPanel(Graphics g, int canvasWidth, int canvasHeight, int startRes)
  {
      AlignmentI da = av.getAlignment();

      int cWidth  =   canvasWidth/av.charWidth;
      int cHeight =  (av.getAlignment().getHeight() + 2)*av.charHeight;

      int  endx   = startRes+cWidth-1;
      int ypos  = 2*av.charHeight;

      while (ypos <= canvasHeight)
      {
        drawScale(g, startRes, endx, ypos);
        drawPanel(g, startRes, endx, 0, da.getHeight(), startRes, 0, ypos);

        ypos += cHeight;
        startRes += cWidth;
        endx = startRes + cWidth - 1;

        if (endx > da.getWidth())
          endx = da.getWidth();
      }

  }


  public void drawPanel(Graphics g,int x1,int x2, int y1, int y2,int startx, int starty,int offset) {


    g.setFont(av.getFont());
    RendererI sr = av.getRenderer();

    /*Vector    pid    = av.getConsensus(false);
    Vector tmpseq = new Vector();
    for (int i = 0; i < av.getAlignment().getHeight(); i++)
	if (!av.getSelection().contains(av.getAlignment().getSequenceAt(i)))
	    tmpseq.addElement(av.getAlignment().getSequenceAt(i));

    if (sr instanceof SequenceRenderer)
	pid    = AAFrequency.calculate(tmpseq,x1,x2);

    else if (sr instanceof GraphRenderer)
	pid = AAFrequency.calculatePID(av.getAlignment().getSequenceAt(0),
				       av.getAlignment().getSequences(),
				       av.getPIDWindow(),x1,x2);

*/

   /* if (y2 > starty && y1 < av.getEndSeq())
    {
       fillBackground(g,
                   Color.red,
                   (x1-startx)*charWidth,
                   offset + AlignmentUtil.getPixelHeight(starty,y1,av.getCharHeight()),
                   (x2-x1+1)*charWidth,
                   offset + AlignmentUtil.getPixelHeight(y1,y2,av.getCharHeight()));
    }*/

    SequenceI nextSeq;
    SequenceGroup group=null;
    SequenceGroup [] groups=null;
    boolean inGroup=false; // are we in a defined group?
    boolean inRB=false; //Are we in a rubberband group?
    int sx=-1, sy=-1, ex=-1, ey=-1;
    int oldStartX=0,oldEndX=0,oldY=0;
    int oldRStartX=0,oldREndX=0,oldRY=0;
    for (int i = y1 ; i < y2 ;i++)
    {
     nextSeq = av.getAlignment().getSequenceAt(i);

     group = av.alignment.findGroup( nextSeq );
     groups = av.alignment.findAllGroups( nextSeq );

     sr.drawSequence(g, nextSeq, groups,x1,x2,
                 (x1 - startx) * av.charWidth,
                 offset + AlignmentUtil.getPixelHeight(starty, i, av.charHeight),
                 av.charWidth,av.charHeight,null, i);

     if(av.showSequenceFeatures)
     {
       fr.drawSequence(g, nextSeq, groups, x1, x2,
                       (x1 - startx) * av.charWidth,
                       offset +
                       AlignmentUtil.getPixelHeight(starty, i, av.charHeight),
                       av.charWidth, av.charHeight, null, i);
     }




     if( group!=null )
     {

        g.setColor(Color.lightGray);

        sx = (group.getStartRes()-startx)*av.charWidth;
        sy = offset + AlignmentUtil.getPixelHeight(starty, i, av.charHeight);
        ex = (group.getEndRes()+1-startx)*av.charWidth;
        ey = offset + AlignmentUtil.getPixelHeight(starty, i+1, av.charHeight);


        if (!inGroup)
        {
          g.drawLine(sx, sy, ex, sy); // Horizontal, top of new box
          inGroup=true;
        }

        g.drawLine(sx, sy, sx, ey ); // vertical line, left hand side
        g.drawLine( ex,sy,ex,ey);    // vertical line, right hand side
        if (i == y2 - 1)// last line of alignment, seal the box
          g.drawLine(sx, ey, ex, ey); // Horizontal, bottom of old box

        oldStartX = sx;
        oldEndX = ex;
        oldY = ey;

     }
     else if(inGroup)
     {
       g.setColor(Color.lightGray);
       g.drawLine(oldStartX, oldY, oldEndX, oldY); // Horizontal, bottom of old box
     }
     else
       inGroup = false;

     group = av.getRubberbandGroup();
     if( group != null && group.sequences.contains(nextSeq) )
     {
         g.setColor(Color.RED.brighter());
         sx = (group.getStartRes() - startx) * av.charWidth;
         sy = offset +
             AlignmentUtil.getPixelHeight(starty, i, av.charHeight);
         ex = (group.getEndRes() + 1 - startx) * av.charWidth;
         ey = offset +
             AlignmentUtil.getPixelHeight(starty, i + 1, av.charHeight);

         if (!inRB)
         {

           g.drawLine(sx, sy, ex, sy); // Horizontal, top of new box
           inRB = true;
         }

         g.drawLine(sx, sy, sx, ey); // vertical line, left hand side
         g.drawLine(ex, sy, ex, ey); // vertical line, right hand side
         if (i == y2 - 1) // last line of alignment, seal the box
           g.drawLine(sx, ey, ex, ey); // Horizontal, bottom of old box

          oldRStartX = sx;
          oldREndX = ex;
          oldRY = ey;
     }
     else if(inRB)
     {
       g.setColor(Color.RED.brighter());
       g.drawLine(oldRStartX, oldRY, oldREndX, oldRY); // Horizontal, bottom of old box
       inRB = false;
     }
     else
       inRB = false;

    }

    /// Highlight search Results once all sequences have been drawn
    if(displaySearch)
    {
      for(int r=0; r<searchResults.length; r+=3)
      {
        int searchSeq = searchResults[r];
        int searchStart = searchResults[r+1];
        int searchEnd = searchResults[r+2];

        if (searchSeq >= y1 && searchSeq < y2)
        {
          SequenceRenderer ssr = (SequenceRenderer) sr;
          ssr.drawHighlightedText(av.getAlignment().getSequenceAt(searchSeq),
                                  searchStart,
                                  searchEnd,
                                  (searchStart - startx) * av.charWidth,
                                  offset +
                                  AlignmentUtil.getPixelHeight(starty, searchSeq,
              av.charHeight),
                                  av.charWidth,
                                  av.charHeight);
        }
      }
    }

  }

  public void fillBackground(Graphics g,Color c, int x1,int y1,int width,int height) {
    g.setColor(c);
    g.fillRect(x1,y1,width,height);
  }

  public int getChunkWidth() {
    return chunkWidth;
  }

  public void highlightSearchResults(int [] results)
  {
    // results are in the order sequence, startRes, endRes
    if(results==null)
      displaySearch = false;
    else
      displaySearch = true;

    searchResults = results;

    paintFlag = true;
    repaint();
  }


}
